// ignore_for_file: public_member_api_docs

import 'package:copy_with_extension/copy_with_extension.dart';
import 'package:flutter/material.dart';
import 'package:flutter_mobx/flutter_mobx.dart';
import 'package:healthrian_visualization_support/'
    'healthrian_visualization_support.dart';

part 'ecg_monitor.g.dart';

_$EcgMonitorCWProxy get EcgWearEcg =>
    EcgMonitor(module: BLEManager().wearEcg12.module).copyWith;

_$EcgMonitorCWProxy get EcgCardioClick =>
    EcgMonitor(module: BLEManager().cardioClick.module).copyWith;

_$EcgMonitorCWProxy get EcgWearT =>
    EcgMonitor(module: BLEManager().wearCardioT.module).copyWith;

@CopyWith()
class EcgMonitor extends StatelessWidget {
  const EcgMonitor({
    super.key,
    required this.module,
    this.onLoadingWidget,
    this.width,
    this.height,
    this.color = Colors.black,
    this.wrapperWidget,
    this.pivotColor = Colors.transparent,
    this.pivotStrokeWidth = 0,
    this.strokeWidth = 1.5,
    this.maxY = 1.5,
    this.minY = -1.5,
    this.backgroundColor = Colors.transparent,
    this.ignoreChannels = const [],
    this.isTappable = false,
    this.isShowGrid = true,
    this.isFiltered = true,
    this.gridColor = Colors.red,
    this.samplingRate = 250,
    this.gridStrokeWidth,
  });

  final ModuleInterface module;

  final Color color;
  final double? width;
  final double? height;
  final Widget? onLoadingWidget;
  final Color pivotColor;
  final double pivotStrokeWidth;

  final double strokeWidth;
  final double minY;
  final double maxY;

  final Color backgroundColor;

  final List<int> ignoreChannels;

  final bool isTappable;
  final bool isShowGrid;

  final Color gridColor;
  final double? gridStrokeWidth;

  final bool isFiltered;
  final int samplingRate;

  //////////////////////

  final Widget Function(List<Widget>)? wrapperWidget;

  //////////////////////

  @override
  Widget build(BuildContext context) {
    final streams = module.ble.streams;
    return Observer(
      key: UniqueKey(),
      builder: (context) {
        return StreamBuilder(
          stream: isFiltered
              ? streams.ecg.filter.plotRT
              : streams.ecg.original.plotRT,
          builder: (context, snapshot) {
            if (!snapshot.hasData) {
              return onLoadingWidget ?? const Text('No Stream');
            }
            final data = snapshot.data!;

            final samplingRate = module.ecg?.samplingRate ?? this.samplingRate;
            final measureTime = data.l2.length / samplingRate;

            final children = [
              for (var i = 0; i < data.list.length; i++)
                if (!ignoreChannels.contains(i))
                  SizedBox(
                    width: width,
                    height: height,
                    child: AspectRatio(
                      aspectRatio: measureTime * 5 / ((maxY - minY) * 2),
                      child: ChannelVisualization(
                        ecgSequence: data.list[i],
                        color: color,
                        pivot: data.pivot,
                        pivotColor: pivotColor,
                        pivotStrokeWidth: pivotStrokeWidth,
                        backgroundColor: backgroundColor,
                        samplingRate: samplingRate,
                        maxY: maxY,
                        minY: minY,
                        strokeWidth: strokeWidth,
                        isTappable: isTappable,
                        countRemoves: data.countRemoves,
                        countUpdates: data.countUpdates,
                        isShowGrid: isShowGrid,
                        gridColor: gridColor,
                        gridStrokeWidth: gridStrokeWidth,
                      ),
                    ),
                  ),
            ];

            if (wrapperWidget == null) {
              return Column(
                children: children,
              );
            }
            return wrapperWidget!(children);
          },
        );
      },
    );
  }
}
